use axum::{
    http::StatusCode,
    response::{IntoResponse, Response},
    Json,
};
use eyre::Report;
use serde::Serialize;
use serde_json::json;
use tracing::error;

pub struct ApiError {
    status: StatusCode,
    code: ErrorCode,
    message: String,
}

impl IntoResponse for ApiError {
    fn into_response(self) -> Response {
        (
            self.status,
            Json(json!({
                "status": self.status.as_u16(),
                "code": self.code,
                "message": self.message,
            })),
        )
            .into_response()
    }
}

#[derive(Serialize)]
#[serde(rename_all = "SCREAMING_SNAKE_CASE")]
pub enum ErrorCode {
    InternalServerError,
}

impl From<sqlx::Error> for ApiError {
    fn from(err: sqlx::Error) -> Self {
        ApiError {
            status: StatusCode::INTERNAL_SERVER_ERROR,
            code: ErrorCode::InternalServerError,
            message: if cfg!(debug_assertions) {
                format!("Database error: {}", err)
            } else {
                "Database error".into()
            },
        }
    }
}

impl From<Report> for ApiError {
    fn from(err: Report) -> Self {
        error!("Error in handler: {}", err);
        ApiError {
            status: StatusCode::INTERNAL_SERVER_ERROR,
            code: ErrorCode::InternalServerError,
            message: if cfg!(debug_assertions) {
                format!("Internal server error: {}", err)
            } else {
                "Internal server error".into()
            },
        }
    }
}
