use std::sync::Arc;

use axum::{Extension, Json};
use eyre::{Context, Result};
use foxchat::http::ApiError;
use serde::{Deserialize, Serialize};
use ulid::Ulid;

use crate::app_state::AppState;

const PASSWORD_COST: u32 = 12;

pub async fn create_user(
    Extension(state): Extension<Arc<AppState>>,
    Json(data): Json<CreateUserRequest>,
) -> Result<Json<CreateUserResponse>, ApiError> {
    let password_hash =
        bcrypt::hash(data.password, PASSWORD_COST).wrap_err("failed to hash password")?;

    let account = sqlx::query!(
        "insert into accounts (id, username, email, password) values ($1, $2, $3, $4) returning id, username",
        Ulid::new().to_string(), data.username, data.email, password_hash)
            .fetch_one(&state.pool)
            .await?;

    Ok(Json(CreateUserResponse {
        id: account.id,
        username: account.username,
    }))
}

#[derive(Deserialize)]
pub struct CreateUserRequest {
    pub username: String,
    pub email: String,
    pub password: String,
}

#[derive(Serialize)]
pub struct CreateUserResponse {
    pub id: String,
    pub username: String,
}
